let colorModelSelected          = new THREE.Vector4(0.02, 0.58, 0.84, 0.5);
let newInstance                 = true;
let ghosting                    = true;
let disableViewerSelectionEvent = false;
let viewerDone                  = false;

let viewer, dmu, markup, markupsvg, curViewerState, restoreMarkupSVG, restoreMarkupState;

let markupStyle = {
    'stroke-width'  : 1.1,
    'font-size'     : 10
    // * defaults got disabled in Mar 2019 due to issues with scaling *
    //    "font-size" : 12,
    //    "stroke-color" : "#FC5A78"
};

let vectorRange  = [ 
    new THREE.Vector4(206/255, 101/255, 101/255, 0.8),
    new THREE.Vector4(224/255, 175/255,  75/255, 0.8), 
    new THREE.Vector4(225/255, 225/255,  84/255, 0.8), 
    new THREE.Vector4(144/255, 216/255,  71/255, 0.8), 
    new THREE.Vector4( 59/255, 210/255,  59/255, 0.8), 
    new THREE.Vector4( 59/255, 197/255, 128/255, 0.8), 
    new THREE.Vector4( 59/255, 186/255, 186/255, 0.8), 
    new THREE.Vector4(104/255, 158/255, 212/255, 0.8), 
    new THREE.Vector4( 81/255, 120/255, 200/255, 0.8), 
    new THREE.Vector4(156/255, 107/255, 206/255, 0.8), 
    new THREE.Vector4(212/255, 103/255, 212/255, 0.8), 
    new THREE.Vector4(206/255,  92/255, 149/255, 0.8)
]; 


// Launch Forge Viewer
function initViewer(data, color, id) {

    if(typeof data === undefined) return;
    else if(data === "") return;

    if(typeof color !== 'undefined') {
        if(color !== null) {
            config.viewer.backgroundColor = color;
        }
    }

    if(typeof id === 'undefined') id = 'viewer';
    else if(id === null) id = 'viewer';

    $('#' + id + '-message').hide();
    $('#' + id + '-processing').hide();
    $('#' + id).show();

    var options = {
        logLevel    : 0,
        env         : 'AutodeskProduction',
        api         : 'derivativeV2',  // for models uploaded to EMEA change this option to 'derivativeV2_EU'
        getAccessToken  : function(onTokenReady) {
            var token = data.token;
            var timeInSeconds = 3600; // Use value provided by Forge Authentication (OAuth) API
            onTokenReady(token, timeInSeconds);
        }
    };

    if(typeof viewer === 'undefined') { 

        Autodesk.Viewing.Initializer(options, function() {

            var htmlDiv = document.getElementById(id);
            viewer = new Autodesk.Viewing.Private.GuiViewer3D(htmlDiv, { modelBrowserStartCollapsed: true, startCollapsed: true });
            viewer.addEventListener(Autodesk.Viewing.VIEWER_STATE_RESTORED_EVENT, onViewerRestore);
            viewer.addEventListener(Autodesk.Viewing.SELECTION_CHANGED_EVENT, onViewerSelectionChanged);

            var startedCode = viewer.start();
            if (startedCode > 0) {
                console.error('Failed to create a Viewer: WebGL not supported.');
                return;
            }

            Autodesk.Viewing.Document.load('urn:'+ data.urn, onDocumentLoadSuccess, onDocumentLoadFailure);
            
        });
    
    } else {

        unloadAllModels();
        newInstance = false;
        Autodesk.Viewing.Document.load('urn:'+ data.urn, onDocumentLoadSuccess, onDocumentLoadFailure);

    } 
    
}
function onDocumentLoadSuccess(doc) {

    viewer.setGhosting(true);
    viewer.setGroundShadow(config.viewer.groundShadow);
    viewer.setGroundReflection(config.viewer.setGroundReflection);
    viewer.setProgressiveRendering(true);

    var viewable = doc.getRoot().getDefaultGeometry();

    if (viewable) {
        // viewer.loadDocumentNode(doc, viewable).then(function(result) {
        viewer.loadDocumentNode(doc, viewable, {globalOffset: {x:0,y:0,z:0}}).then(function(result) {
            viewer.setBackgroundColor(config.viewer.backgroundColor, config.viewer.backgroundColor, config.viewer.backgroundColor, config.viewer.backgroundColor, config.viewer.backgroundColor, config.viewer.backgroundColor);
            viewerDone = true;
            initViewerDone(newInstance);
        }).catch(function(err) {
            console.log(err);
        });
    }
}
function onDocumentLoadFailure() {
    console.error('Failed fetching Forge manifest');
}
function onViewerRestore(event) {
     
    markup.unloadMarkupsAllLayers();
    
    if(markupsvg !== "") {
        markup.show();
        markup.loadMarkups(markupsvg, "review");
        
    }
    
}
function initViewerDone() {
    $('#viewer-progress').hide();
}
function onViewerSelectionChanged(event) {}



// Resize Viewer
function viewerResize(delay) {

    if(!viewerDone) return;

    if(typeof delay === 'undefined') delay = 250;

    setTimeout(function() { viewer.resize(); }, delay);

}



// Launch Forge Aggregated View
// function initDMU(data, color, id) {

//     if(typeof data === undefined) return;
//     else if(data === "") return;

//     if(typeof color !== 'undefined') {
//         if(color !== null) {
//             backgroundColor = color;
//         }
//     }

//     if(typeof id === 'undefined') id = 'viewer';
//     else if(id === null) id = 'viewer';

//     $('#' + id + '-message').hide();
//     $('#' + id).show();
//     $('#' + id + '-progress').hide();

//     var options = {
//         logLevel : 0,
//         env: 'AutodeskProduction',
//         api: 'derivativeV2',  // for models uploaded to EMEA change this option to 'derivativeV2_EU'
//         getAccessToken: function(onTokenReady) {
//             var token = data.token;
//             var timeInSeconds = 3600; // Use value provided by Forge Authentication (OAuth) API
//             onTokenReady(token, timeInSeconds);
//         }
//     };

//     if(typeof dmu === 'undefined') { 

//         dmu = new Autodesk.Viewing.AggregatedView();

//         Autodesk.Viewing.Initializer(options, function onInitialized() {
//             // Get the Viewer DIV
//             var htmlDiv = document.getElementById(id);
        
//             // Initialize the AggregatedView view
//             dmu.init(htmlDiv, options).then(function () {
//                 Autodesk.Viewing.Document.load('urn:'+ data.urn, (doc) => {
//                     var nodes = doc.getRoot().search({ type: 'geometry' });
//                     dmu.setNodes(nodes[0]);
//                 }, (errorCode, errorMsg, messages) => {});
//             });
//         });




//         // var htmlDiv = document.getElementById(id);

//         // await dmu.init(htmlDiv, options)

//         // Autodesk.Viewing.Initializer(options, function() {

            
            
//         //     dmu.addEventListener(Autodesk.Viewing.VIEWER_STATE_RESTORED_EVENT, onViewerRestore);
//         //     dmu.addEventListener(Autodesk.Viewing.SELECTION_CHANGED_EVENT, onSelectionChanged);

//         //     var startedCode = dmu.start();
//         //     if (startedCode > 0) {
//         //         console.error('Failed to create a Viewer: WebGL not supported.');
//         //         return;
//         //     }

//         //     Autodesk.Viewing.Document.load('urn:'+ data.urn, onDocumentLoadSuccess, onDocumentLoadFailure);
            
//         // });
    
//     } else {

//         // unloadAll();
//         // newInstance = false;
//         // Autodesk.Viewing.Document.load('urn:'+ data.urn, onDocumentLoadSuccess, onDocumentLoadFailure);

//     } 
    
// }
// function insertModels(list) {

//     console.log(list);

// let tasks = [];

//     for(item of list) {

//         // models.forEach( md => tasks.push( loadManifest( md.urn ) ) );
//         // docs.push(Autodesk.Viewing.Document.load('urn:' + item));
//         tasks.push(loadManifest('urn:' + item));

//     }

//     console.log(tasks.length);
//     console.log(tasks);

//     Promise.all(tasks).then( docs =>  Promise.resolve( docs.map( doc => {
//       const bubbles = doc.getRoot().search({type:'geometry', role: '3d'});
//       const bubble = bubbles[0];
//       if( !bubble ) return null;

//       return bubble;
//     }))).then( bubbles => dmu.setNodes( bubbles ) );
// // });


//     // console.log(docs.length);
//     // console.log(docs);

//     // const bubbleNodes = [];
//     // // docs is an array of loaded documents
//     // docs.forEach((doc) => {
//     //     var nodes = doc.getRoot().search({ type: 'geometry' });
//     //     bubbleNodes.push(nodes[0]);
//     // });
//     // dmu.setNodes(bubbleNodes);

// }

// function loadManifest( documentId ) {
//     return new Promise(( resolve, reject ) => {
//     //   const onDocumentLoadSuccess = ( doc ) => {
//     //     doc.downloadAecModelData(() => resolve(doc));
//     //   };
//       Autodesk.Viewing.Document.load( documentId, function() {
//         return resolve();
//       }, reject );
//     });
//   }









// Add & unload models to session
function viewerAddModel(model) {
    viewerAddModels([model]);
    // }]);
}
function viewerAddModels(models) {
    for(model of models) addModel(model);
}
// function addModel(urn, transX, transY, transZ, rotX, rotY, rotZ) {
function addModel(model) {

    // see https://aps.autodesk.com/blog/loading-multiple-models-forge-viewer-v7 about transformation
    
    if(typeof model.offsetX === 'undefined') model.offsetX = 0;
    if(typeof model.offsetY === 'undefined') model.offsetY = 0;
    if(typeof model.offsetZ === 'undefined') model.offsetZ = 0;

    if(typeof model.angleX === 'undefined') model.angleX = 0;
    if(typeof model.angleY === 'undefined') model.angleY = 0;
    if(typeof model.angleZ === 'undefined') model.angleZ = 0;

    // console.log(model);

    Autodesk.Viewing.Document.load('urn:'+ model.urn, function(doc) {


        // const rotation = new THREE.Matrix4().makeRotationY(0);
        // const rotation = new THREE.Matrix4().makeRotationX(Number(model.angleX)).makeRotationY(Number(model.angleY)).makeRotationZ(Number(model.angleZ));
        // const rotation = new THREE.Matrix4().makeRotationX(Number(model.angleX));
        // const rotation = new THREE.Matrix4().makeRotationY(Number(model.angleY));
        // const rotation = new THREE.Matrix4().makeRotationZ(Number(model.angleZ));
        // const translation = new THREE.Matrix4().makeTranslation(0, 0, 0);
        // const rotation = new THREE.Matrix4().makeRotationX(Number(model.angleX));// works
        const rotation = new THREE.Matrix4().makeRotationX(Number(model.angleX)).makeRotationY(Number(model.angleY));
        const translation = new THREE.Matrix4().makeTranslation(Number(model.offsetX), Number(model.offsetY), Number(model.offsetZ));
        // const translation = new THREE.Matrix4();

        // console.log(Number(model.offsetX) + ',' + Number(model.offsetY) + ',' + Number(model.offsetZ) + '    /     ' + Number(model.angleX) + ',' + Number(model.angleY) + ',' + Number(model.angleZ));
        // console.log(Number(model.angleY));
        // console.log(Number(model.angleZ));

        var viewable = doc.getRoot().getDefaultGeometry();
        viewer.loadDocumentNode(doc, viewable, {
            preserveView: false,
            // keepCurrentModels: true,
            // placementTransform: (new THREE.Matrix4()).setPosition({x:-1000,y:1000,z:0}),
            // placementTransform: (new THREE.Matrix4()).makeRotationY(90).makeRotationZ(90).setPosition({x:-1000,y:1000,z:0}),
            // placementTransform: (new THREE.Matrix4()).makeRotationX(Number(model.angleX)).makeRotationY(Number(model.angleY)).makeRotationZ(Number(model.angleZ)).setPosition({x:Number(model.offsetX), y:Number(model.offsetY), z:Number(model.offsetZ)}),
            // placementTransform: translation,   // works
            // placementTransform: rotation, // fails
            // placementTransform: new THREE.Matrix4().makeRotationX(Number(model.angleX)), // works
            // placementTransform: new THREE.Matrix4().makeRotationX(Number(model.angleX)).makeTranslation(Number(model.offsetX), Number(model.offsetY), Number(model.offsetZ)), // fails
            placementTransform: rotation.multiply(translation), // works
            keepCurrentModels: true,
            globalOffset: {x:0,y:0,z:0}
        }).then(function(result) {
            // viewer.showAll();
        }).catch(function(err) {
            console.log(err);
        });

    }, onDocumentLoadFailure);

}
function viewerUnloadModel(urn) {
    viewerUnloadModels([urn]);
}
function viewerUnloadModels(urns) {
    for(urn of urns) unloadModel(urn);
}
function unloadModel(urn) {

    let models = viewer.impl.modelQueue().getModels();

    for(model of models) {
        if(model.myData.urn === urn) viewer.unloadModel(model);
    }

}


// Close all models currently in viewer
function unloadAllModels() {

    if(viewer === null) return;
    if(typeof viewer === 'undefined') return;

    let models = viewer.impl.modelQueue().getModels();

    for(model of models) viewer.impl.unloadModel(model);

}


// Select and focus on selected item
function viewerSelectModel(partNumber, fitToView) {

    viewerSelectModels([partNumber], fitToView);

}
function viewerSelectModels(partNumbers, fitToView) {

    if(!viewerDone) return;

    disableViewerSelectionEvent = true;
    viewer.hideAll();

    if(typeof fitToView === 'undefined') fitToView = false;

    let instances   = viewer.model.getInstanceTree();
    let dbIds       = [];

    for(var i = 1; i < instances.objectCount; i++) {

        viewer.model.getProperties(i, function(data) { 

            let partNumberValues = [];

            for(property of data.properties) {
                if(config.viewer.partNumberProperties.indexOf(property.attributeName) > -1) {

                    let value = property.displayValue;
                        value = value.split(':')[0];

                    partNumberValues.push(value);

                }
            }

            if(partNumberValues.length > 0) {
                for(partNumberValue of partNumberValues) {
                    for(partNumber of partNumbers) {
                        if(partNumberValue.indexOf(partNumber) === 0) {
                            dbIds.push(data.dbId);
                            viewer.show(data.dbId);
                            viewer.setThemingColor(data.dbId, colorModelSelected, null, true );
                            if(fitToView) viewer.fitToView(dbIds);
                            break;
                        }
                    }
                }
            }
            
        })
    }

    disableViewerSelectionEvent = false;

}
function viewerSelectModelNew(selected, fitToView) {

    let partNumbers = [selected];
    let dbIds = [];

    disableViewerSelectionEvent = true;
    viewer.hideAll();

    if(typeof fitToView === 'undefined') fitToView = false;

    // let instances   = viewer.model.getInstanceTree();
//     let dbIds       = [];


// let dbId = 1;


// for(var i = 1; i < instances.objectCount; i++) {

//     for(model of viewer.getAllModels()) {

//         model.getProperties(i, function(data) { 
//         data.dbId = dbId++;
//         });

//     }
// }


    // let models = viewer.getAllModels();

    let models = viewer.getAllModels();


    for(let x = 0; x < models.length; x++) {

        let model = models[x];
        let instances   = model.getInstanceTree();

        for(var i = 1; i < instances.objectCount; i++) {
                
            model.getProperties(i, function(data) { 

                let partNumberValues = [];

                for(property of data.properties) {
                    if(config.viewer.partNumberProperties.indexOf(property.attributeName) > -1) {

                        let value = property.displayValue;
                            value = value.split(':')[0];

                        partNumberValues.push(value);

                    }
                }

                if(partNumberValues.length > 0) {
                    for(partNumber of partNumberValues) {
                        for(number of partNumbers) {
                            if(partNumber.indexOf(number) === 0) {
                                dbIds.push(data.dbId);  
                                viewer.show(data.dbId, model);
                                viewer.setThemingColor(data.dbId, config.viewer.colorModelSelected, model, true );
                                if(fitToView) viewer.fitToView(dbIds);
                                break;
                            }
                        }
                    }
                }
                
            })
        }
    }

    disableViewerSelectionEvent = false;

}
function viewerSelectAll(fitToView) {

    disableViewerSelectionEvent = true;
    
    if(typeof fitToView === 'undefined') fitToView = true;
    
    viewer.showAll();
    
    let dbIds  = [];
    let models = viewer.getAllModels();

    for(let x = 0; x < models.length; x++) {

        let model     = models[x];
        let instances = model.getInstanceTree();

        for(var i = 1; i < instances.objectCount; i++) {
                
            model.getProperties(i, function(data) { 

                dbIds.push(data.dbId);  
                viewer.setThemingColor(data.dbId, colorModelSelected, model, true);
                
            })
        }
    }

    if(fitToView) viewer.setViewFromFile();

    disableViewerSelectionEvent = false;

}



// Hide given model item
function viewerHideModel(partNumber) {

    viewerHideModels([partNumber]);

}
function viewerHideModels(partNumbers) {

    let instances   = viewer.model.getInstanceTree();
    let dbIds       = [];

    disableViewerSelectionEvent = true;

    for(var i = 1; i < instances.objectCount; i++) {

        viewer.model.getProperties(i, function(data) { 

            let partNumberValues = [];

            for(property of data.properties) {
                if(config.viewer.partNumberProperties.indexOf(property.attributeName) > -1) {

                    let value = property.displayValue;
                        value = value.split(':')[0];

                    partNumberValues.push(value);

                }
            }

            if(partNumberValues.length > 0) {
                for(partNumber of partNumberValues) {
                    for(number of partNumbers) {
                        if(partNumber.indexOf(number) === 0) {
                            viewer.hide(data.dbId);
                            break;
                        }
                    }
                }
            }
            
        })
    }

    disableViewerSelectionEvent = false;

}


// Unhide given model item
function viewerUnhideModel(partNumber) {

    viewerUnhideModels([partNumber]);

}
function viewerUnhideModels(partNumbers) {

    let instances   = viewer.model.getInstanceTree();
    let dbIds       = [];

    for(var i = 1; i < instances.objectCount; i++) {

        viewer.model.getProperties(i, function(data) { 

            let partNumberValues = [];

            for(property of data.properties) {
                if(config.viewer.partNumberProperties.indexOf(property.attributeName) > -1) {

                    let value = property.displayValue;
                        value = value.split(':')[0];

                    partNumberValues.push(value);

                }
            }

            if(partNumberValues.length > 0) {
                for(partNumber of partNumberValues) {
                    for(number of partNumbers) {
                        if(partNumber.indexOf(number) === 0) {
                            viewer.show(data.dbId);
                            break;
                        }
                    }
                }
            }
            
        })
    }

}


// Reset viewer / deselect all
function viewerResetSelection(resetView) {

    if(!viewerDone) return;

    viewer.showAll();
    viewer.clearSelection();
    viewer.clearThemingColors();
    
    if(resetView !== null) {
        if(resetView) {
            viewer.setViewFromFile();
        }
    }

}


// Set custom colors for multiple records
function viewerSetColor(partNumber, color, fitToView) {

    viewerSetColors([partNumber], color, fitToView);

}
function viewerSetColors(partNumbers, color, fitToView) {

    if(typeof viewer === 'undefined') return;
    if(typeof partNumbers === 'undefined') return;
    if(partNumbers.length === 0) return;

    if(typeof fitToView === 'undefined') fitToView = false;

    let vector      = new THREE.Vector4(color[0], color[1], color[2], color[3]);
    let instances   = viewer.model.getInstanceTree();
    let dbIds       = [];

    for(var i = 1; i < instances.objectCount; i++) {

        viewer.model.getProperties(i, function(data) { 

            let partNumberValues = [];

            for(property of data.properties) {
                if(config.viewer.partNumberProperties.indexOf(property.attributeName) > -1) {
                    let value = property.displayValue;
                        value = value.split(':')[0];

                    partNumberValues.push(value);
                }
            }

            if(partNumberValues.length > 0) {
                for(partNumber of partNumberValues) {
                    for(number of partNumbers) {
                        if(partNumber.indexOf(number) > -1) {
                            viewer.setThemingColor(data.dbId, vector, null, true);
                            // viewer.show(data.dbId);
                            if(fitToView) viewer.fitToView(dbIds);
                        }
                    }
                }
            }

        })

    }

}


// Reset all custom colors
function viewerResetColors() {

    if(!viewerDone) return;

    if(typeof viewer !== 'undefined') {
        if(viewer.started) {
            viewer.clearThemingColors();
        }
    }

}


// Close markup view
// function viewerLeaveMarkupMode() {

//     if(typeof markup === 'undefined') return;

//     markup.leaveEditMode();
//     markup.hide();

// }



// Custom Controls: Reset
function viewerAddResetButton() {

    let customToolbar = new Autodesk.Viewing.UI.ControlGroup('custom-toolbar-reset');

    let buttonReset = addCustomControl(customToolbar, 'button-reset-selection', 'icon-close', 'Reset selection and show all models');
        buttonReset.onClick = function() { 
            viewer.showAll();
            viewerResetColors();
            viewerSelectionResetDone();
        };
        
    viewer.toolbar.addControl(customToolbar);

}
function viewerSelectionResetDone() {}



// Custom Controls : Ghosting
function viewerAddGhostingToggle() {

    let newToolbar = new Autodesk.Viewing.UI.ControlGroup('my-custom-toolbar-ghosting');

    let buttonOff = addCustomControl(newToolbar, 'button-toggle-ghosting-off', 'icon-hide', 'Enable ghosting mode');
        buttonOff.onClick = function(e) { 
            viewer.setGhosting(true);
            $('#my-custom-toolbar-ghosting').addClass('no-ghosting');
            $('#my-custom-toolbar-ghosting').removeClass('ghosting');
        };

    let buttonOn = addCustomControl(newToolbar, 'button-toggle-ghosting-on', 'icon-show', 'Disable ghosting mode');
        buttonOn.onClick = function(e) { 
            viewer.setGhosting(false);
            $('#my-custom-toolbar-ghosting').removeClass('no-ghosting');
            $('#my-custom-toolbar-ghosting').addClass('ghosting');
        };

    viewer.toolbar.addControl(newToolbar);

    $('#my-custom-toolbar-ghosting').addClass('no-ghosting');

}
function addCustomControl(toolbar, id, icon, tooltip) {

    let newButton = new Autodesk.Viewing.UI.Button(id);
        newButton.addClass('icon');
        newButton.setIcon(icon);
        newButton.setToolTip(tooltip);

    toolbar.addControl(newButton);

    return newButton;

}


// Custom Controls : Standard views
function viewerAddViewsToolbar() {

    let newToolbar  = new Autodesk.Viewing.UI.ControlGroup('my-custom-toolbar-views');
    
    addCustomViewControl(newToolbar, 'my-view-home-button'  , 'home'  , 'icon-home', 'Home');
    addCustomViewControl(newToolbar, 'my-view-front-button' , 'front' , 'icon-north-east', 'Front View');
    addCustomViewControl(newToolbar, 'my-view-back-button'  , 'back'  , 'icon-south-west', 'Back View');
    addCustomViewControl(newToolbar, 'my-view-left-button'  , 'left'  , 'icon-east', 'Left View');
    addCustomViewControl(newToolbar, 'my-view-right-button' , 'right' , 'icon-west', 'Right View');
    addCustomViewControl(newToolbar, 'my-view-top-button'   , 'top'   , 'icon-south', 'Top View');
    addCustomViewControl(newToolbar, 'my-view-bottom-button', 'bottom', 'icon-north', 'Bottom View');

    viewer.toolbar.addControl(newToolbar);

}
function addCustomViewControl(toolbar, id, view, icon, tooltip) {
    
    var button = new Autodesk.Viewing.UI.Button(id);
        button.addClass('icon');
        button.setIcon(icon);
        button.setToolTip(tooltip);
    
    if(view === "home") {
        button.onClick = function(e) { viewer.setViewFromFile(); };
    } else {
        button.onClick = function(e) { 
            let viewcuiext = viewer.getExtension('Autodesk.ViewCubeUi');
                viewcuiext.setViewCube(view);
        };
    }
    
    toolbar.addControl(button);
    
}


// Custom Controls : Note
function viewerAddNoteControls() {

    let elemNoteToolbar = $('<div></div>');
        elemNoteToolbar.attr('id', 'viewer-note-toolbar');
        elemNoteToolbar.addClass('hidden');
        elemNoteToolbar.appendTo($('#viewer'));

    let elemNoteGroup = addMarkupControlGroup(elemNoteToolbar, 'Note');

    let elemInput = $('<textarea></textarea>');
        elemInput.addClass('viewer-note');
        elemInput.attr('id', 'viewer-note');
        elemInput.appendTo(elemNoteGroup);

}



// Custom Controls : Markup
function viewerAddMarkupControls() {

    let elemMarkupToolbar = $('<div></div>');
        elemMarkupToolbar.attr('id', 'viewer-markup-toolbar');
        elemMarkupToolbar.addClass('hidden');
        elemMarkupToolbar.addClass('set-defaults');
        elemMarkupToolbar.appendTo($('#viewer'));

    let elemMarkupGroupColors = addMarkupControlGroup(elemMarkupToolbar, 'Color');

    addMarkupColorControl(elemMarkupGroupColors, 'FB5A79');
    addMarkupColorControl(elemMarkupGroupColors, 'FBE235');
    addMarkupColorControl(elemMarkupGroupColors, '3694FB');
    // addMarkupColorControl(elemMarkupGroupColors, '8CE5FC');
    addMarkupColorControl(elemMarkupGroupColors, '68E759');


    let elemMarkupGroupWidth = addMarkupControlGroup(elemMarkupToolbar, 'Width');

    addMarkupWidthControl(elemMarkupGroupWidth, '1', 1);
    addMarkupWidthControl(elemMarkupGroupWidth, '2', 5);
    addMarkupWidthControl(elemMarkupGroupWidth, '3', 12);
    addMarkupWidthControl(elemMarkupGroupWidth, '4', 25);
    addMarkupWidthControl(elemMarkupGroupWidth, '5', 40);

    let elemMarkupGroupShapes = addMarkupControlGroup(elemMarkupToolbar, 'Shape');

    addMarkupShapeControl(elemMarkupGroupShapes, 'arrow', 'trending_flat');
    addMarkupShapeControl(elemMarkupGroupShapes, 'circle', 'radio_button_unchecked');
    addMarkupShapeControl(elemMarkupGroupShapes, 'rectangle', 'crop_square');
    addMarkupShapeControl(elemMarkupGroupShapes, 'cloud', 'water');
    addMarkupShapeControl(elemMarkupGroupShapes, 'freehand', 'draw');
    addMarkupShapeControl(elemMarkupGroupShapes, 'text', 'text_fields');

    let elemMarkupGroupActions = addMarkupControlGroup(elemMarkupToolbar, 'Actions');

    addMarkupActionControl(elemMarkupGroupActions, true, 'undo', 'markup.undo();');
    addMarkupActionControl(elemMarkupGroupActions, true, 'redo', 'markup.redo();');
    addMarkupActionControl(elemMarkupGroupActions, false, 'Clear', 'markup.clear();');
    addMarkupActionControl(elemMarkupGroupActions, false, 'Close', 'viewerLeaveMarkupMode();');


    let elemMarkupImage = $('<canvas>');
        elemMarkupImage.attr('id', 'viewer-markup-image');
        elemMarkupImage.addClass('hidden');
        elemMarkupImage.appendTo($('body'));


    let newToolbar = new Autodesk.Viewing.UI.ControlGroup('my-custom-markup-toolbar');
    let newButton  = addCustomControl(newToolbar, 'my-markup-button', 'icon-markup', 'Markup');

    newButton.onClick = function() {

        if(restoreMarkupState !== '') {

            markupsvg = restoreMarkupSVG;
            viewer.restoreState(restoreMarkupState, null, true);

        }

        markup.enterEditMode();
        markup.show();
            
        if($('#viewer-markup-toolbar').hasClass('set-defaults')) {
            $('.viewer-markup-toggle.color').first().click();
            $('.viewer-markup-toggle.width').first().click();
            $('.viewer-markup-toggle.shape').first().click();
            $('#viewer-markup-toolbar').removeClass('set-defaults');
        } else {
            $('.viewer-markup-toggle.color.selected').click();
            $('.viewer-markup-toggle.width.selected').click();
            $('.viewer-markup-toggle.shape.selected').click();
        }
        
        $('#viewer-markup-toolbar').toggleClass('hidden');

        let elemNoteControl = $('#viewer-note-toolbar');

        if(elemNoteControl.length > 0) elemNoteControl.toggleClass('hidden');

    };

    viewer.toolbar.addControl(newToolbar);
    
    var promise = viewer.loadExtension('Autodesk.Viewing.MarkupsCore');
    
    promise.then(function(extension){ markup = extension; });

}
function addMarkupControlGroup(elemParent, label) {

    let elemGroup = $('<div></div>');
        elemGroup.addClass('viewer-markup-toolbar-group');
        elemGroup.appendTo(elemParent);

    let elemGroupLabel = $('<div></div>');
        elemGroupLabel.addClass('viewer-markup-toolbar-group-label');
        elemGroupLabel.html(label);
        elemGroupLabel.appendTo(elemGroup);

    return elemGroup;

}
function addMarkupColorControl(elemParent, color) {

    let elemControl = $('<div></div>');
        elemControl.addClass('viewer-markup-toggle');
        elemControl.addClass('color');
        elemControl.css('background', '#' + color);
        elemControl.attr('data-color', color);
        elemControl.appendTo(elemParent);

    elemControl.click(function() {
        markupStyle['stroke-color'] = '#' + $(this).attr('data-color');
        markup.setStyle(markupStyle);
        $(this).siblings().removeClass('selected');
        $(this).addClass('selected');
    });

}
function addMarkupWidthControl(elemParent, label, width) {

    let elemControl = $('<div></div>');
        elemControl.addClass('viewer-markup-toggle');
        elemControl.addClass('width');
        elemControl.html(label);
        elemControl.attr('data-width', width);
        elemControl.appendTo(elemParent);

    elemControl.click(function() {
        markupStyle['stroke-width'] = $(this).attr('data-width');
        markup.setStyle(markupStyle);
        $(this).siblings().removeClass('selected');
        $(this).addClass('selected');
    });

}
function addMarkupShapeControl(elemParent, shape, icon) {

    let elemControl = $('<div></div>');
        elemControl.addClass('viewer-markup-toggle');
        elemControl.addClass('shape');
        elemControl.addClass('icon');
        elemControl.attr('data-shape', shape);
        elemControl.html(icon);
        elemControl.appendTo(elemParent);

    elemControl.click(function() {

        let shape = $(this).attr('data-shape');
        let mode;

        switch (shape) {

            case 'arrow':       mode = new Autodesk.Viewing.Extensions.Markups.Core.EditModeArrow(markup);      break;
            case 'circle':      mode = new Autodesk.Viewing.Extensions.Markups.Core.EditModeCircle(markup);     break;
            case 'cloud':       mode = new Autodesk.Viewing.Extensions.Markups.Core.EditModeCloud(markup);      break;
            case 'freehand':    mode = new Autodesk.Viewing.Extensions.Markups.Core.EditModeFreehand(markup);   break;
            case 'rectangle':   mode = new Autodesk.Viewing.Extensions.Markups.Core.EditModeRectangle(markup);  break;
            case 'text':        mode = new Autodesk.Viewing.Extensions.Markups.Core.EditModeText(markup);       break;
            case 'polycloud':   mode = new Autodesk.Viewing.Extensions.Markups.Core.EditModePolycloud(markup);  break;
            case 'polyline':    mode = new Autodesk.Viewing.Extensions.Markups.Core.EditModePolyline(markup);   break;
        }

        markup.changeEditMode(mode);
        markup.setStyle(markupStyle);
        $(this).siblings().removeClass('selected');
        $(this).addClass('selected');

    });

}
function addMarkupActionControl(elemParent, icon, content, script) {

    let elemControl = $('<div></div>');
        elemControl.addClass('viewer-markup-button');
        elemControl.html(content);
        elemControl.attr('onclick', script);
        elemControl.appendTo(elemParent);
        
    if(icon) elemControl.addClass('icon');

}
function viewerLeaveMarkupMode() {

    let elemNoteControl = $('#viewer-note-toolbar');

    if(elemNoteControl.length > 0) {
        elemNoteControl.toggleClass('hidden');
        closedViewerMarkup(markup.generateData(), viewer.getState());
    }

    $('#viewer-markup-toolbar').addClass('hidden');

    if(typeof markup === 'undefined') return;

    markup.leaveEditMode();
    markup.hide();

    restoreMarkupSVG   = '';
    restoreMarkupState = '';

}
function viewerCaptureScreenshot(callback) {
   
    var screenshot  = new Image();
    var imageWidth  = viewer.container.clientWidth;
    var imageHeight = viewer.container.clientHeight;

    screenshot.onload = function () {
            
        let canvas          = document.getElementById('viewer-markup-image');
            canvas.width    = viewer.container.clientWidth;
            canvas.height   = viewer.container.clientHeight;

        let context = canvas.getContext('2d');
            context.clearRect(0, 0, canvas.width, canvas.height);
            context.drawImage(screenshot, 0, 0, canvas.width, canvas.height); 
            
        if(!$('#viewer-markup-toolbar').hasClass('hidden')) {
            markup.renderToCanvas(context, function() {
                callback();
            });
        } else {
            callback();
        }
            
    }
            
    viewer.getScreenShot(imageWidth, imageHeight, function (blobURL) {
        screenshot.src = blobURL;
    });

}